@@ -1,29 +1,41 @@
 """Module containing the plotting functions for the ds class."""
 import copy
 from dataclasses import dataclass, field
-from typing import Optional, Tuple, List, Any, Dict, Callable
+from typing import Optional, Tuple, List, Any, Dict, Callable, Union
 
 import matplotlib.patches as mpatches
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
 from matplotlib.lines import Line2D
+import matplotlib as mpl
+import seaborn as sns
 
 from paramaterial.plug import DataItem, DataSet
+from paramaterial.preparing import experimental_matrix
 
 
 def configure_plt_formatting():
-    import matplotlib as mpl
     plt.style.use('seaborn-dark')
+    # mpl.rcParams['axes.facecolor'] = '#f0e6e6'
     mpl.rcParams['text.usetex'] = False
     mpl.rcParams['text.latex.preamble'] = r'\usepackage{amsmath} \usepackage{amssymb}'
     mpl.rcParams["font.family"] = "Times New Roman"
-    plt.rc('font', size=11)
-    plt.rc('axes', titlesize=11, labelsize=11)
-    plt.rc('xtick', labelsize=9)
-    plt.rc('ytick', labelsize=9)
-    plt.rc('legend', fontsize=9)
-    plt.rc('figure', titlesize=13)
+    FONTSIZE = 11
+    plt.rc('font', size=FONTSIZE)
+    plt.rc('axes', titlesize=FONTSIZE, labelsize=FONTSIZE)
+    plt.rc('xtick', labelsize=FONTSIZE - 1)
+    plt.rc('ytick', labelsize=FONTSIZE - 1)
+    plt.rc('legend', fontsize=FONTSIZE - 1)
+    plt.rc('figure', titlesize=FONTSIZE + 1)
+    mpl.rcParams.update({"axes.grid": True})
+    # cmap = mpl.colors.LinearSegmentedColormap.from_list("", ["white", (85/255, 49/255, 0)])
+    # mpl.rcParams['axes.facecolor'] = cmap(0.1)
+    # mpl.rcParams['legend.facecolor'] = "white"
+    # mpl.rcParams["grid.linewidth"] = 2
+    # mpl.rcParams["text.color"] = (40/255, 40/255, 40/255)
+    # cmap = mpl.colors.LinearSegmentedColormap.from_list("", ["white", (0.2124, 0.3495, 0.1692)])
+    # mpl.rcParams["axes.facecolor"]= cmap(0.1)
 
 
 configure_plt_formatting()
@@ -46,16 +58,17 @@ class Styler:
     cmap: str = 'plasma'
     handles: Optional[List[mpatches.Patch]] = None
     linestyles: List[str] = field(default_factory=lambda: ['-', '--', ':', '-.'])
-    markers: List[str] = field(default_factory=lambda: ['o', 's', 'v', '^'])
-    color_dict: Optional[Dict[str|int|float, str]] = None
-    linestyle_dict: Optional[Dict[str|int|float, str]] = None
-    marker_dict: Optional[Dict[str|int|float, str]] = None
+    markers: List[str] = field(
+        default_factory=lambda: ['s', 'H', 'd', 'v', 'D', 'p', 'X', 'o', 'd', 'h', 'H', '8', 'P', 'x'])
+    color_dict: Optional[Dict[Union[str, int, float], str]] = None
+    linestyle_dict: Optional[Dict[Union[str, int, float], str]] = None
+    marker_dict: Optional[Dict[Union[str, int, float], str]] = None
     plot_kwargs: Dict[str, Any] = field(default_factory=lambda: dict())
     styled_ds: Optional[DataSet] = None
 
     def __post_init__(self):
         self.plot_kwargs['legend'] = False
-
+        self.plot_kwargs.update({'markeredgecolor': 'white', 'markersize': 8})
         # todo: use pandas in-built color bar
         # if self.cbar:
         #     self.plot_kwargs['cmap'] = self.cmap
@@ -96,11 +109,17 @@ class Styler:
         if self.styled_ds is None:
             raise ValueError('The styler must be styled to a ds before plotting.')
 
-        configure_plt_formatting()
+        # configure_plt_formatting()
         formatters = dict()
 
         if self.color_by is not None:
-            formatters['color'] = self.color_dict[di.info[self.color_by]]
+            if di.info[self.color_by] not in self.color_dict.keys():
+                try:
+                    formatters['color'] = self.color_dict[float(di.info[self.color_by])]
+                except ValueError:
+                    formatters['color'] = plt.gca()._get_lines.get_next_color()
+            else:
+                formatters['color'] = self.color_dict[di.info[self.color_by]]
             if all(str(x).isnumeric() for x in self.color_dict.keys()):
                 formatters['zorder'] = di.info[self.color_by]
         else:
@@ -124,21 +143,23 @@ class Styler:
             return handles
 
         if self.color_by_label is not None:
-            handles.append(mpatches.Patch(label=self.color_by_label, alpha=0))
+            handles.append(mpatches.Patch(label=self.color_by_label.title(), alpha=0))
 
         if self.color_by is not None:
             for color_val in ds.info_table[self.color_by].unique():
+                if color_val not in self.color_dict.keys():
+                    color_val = float(color_val)
                 handles.append(Line2D([], [], label=color_val, color=self.color_dict[color_val], marker='o', ls=''))
 
         if self.linestyle_by_label is not None:
-            handles.append(mpatches.Patch(label='\n' + self.linestyle_by_label, alpha=0))
+            handles.append(mpatches.Patch(label='\n' + self.linestyle_by_label.title(), alpha=0))
 
         if self.linestyle_by is not None:
             for ls_val in ds.info_table[self.linestyle_by].unique():
                 handles.append(Line2D([], [], label=ls_val, ls=self.linestyle_dict[ls_val], c='k', marker=''))
 
         if self.marker_by_label is not None:
-            handles.append(mpatches.Patch(label='\n' + self.marker_by_label, alpha=0))
+            handles.append(mpatches.Patch(label='\n' + self.marker_by_label.title(), alpha=0))
 
         if self.marker_by is not None:
             for marker_val in ds.info_table[self.marker_by].unique():
@@ -153,6 +174,8 @@ def dataset_plot(
         ax: Optional[plt.Axes] = None,
         fill_between: Optional[Tuple[str, str]] = None,
         plot_legend: bool = True,
+        handletextpad: float = 0.05,
+        labelspacing: float = 0.1,
         **kwargs
 ) -> plt.Axes:
     """Make a single combined plot from the data of every dataitem in the dataset using pandas.DataFrame.plot.
@@ -168,12 +191,16 @@ def dataset_plot(
     Returns: The axis the plot was made on.
     """
     if ax is None:
-        fig, (ax) = plt.subplots(1, 1, figsize=kwargs.get('figsize', (6, 4)))
+        fig, (ax) = plt.subplots(1, 1, figsize=kwargs.get('figsize', (4, 3)))
     kwargs['ax'] = ax
 
     if ax.get_legend() is not None and plot_legend:
         ax.get_legend().remove()
 
+    if styler is None:
+        styler = Styler()
+        styler.style_to(ds)
+
     kwargs = {**styler.plot_kwargs, **kwargs}
 
     # plot the dataitems
@@ -188,8 +215,8 @@ def dataset_plot(
     # add the legend
     handles = styler.legend_handles(ds)
     if len(handles) > 0 and plot_legend:
-        ax.legend(handles=handles, loc='best', frameon=True, markerfirst=False,
-                  handletextpad=0.05)  # , labelspacing=0.1)
+        ax.legend(handles=handles, loc='best', frameon=False, markerfirst=True, handletextpad=handletextpad,
+                  labelspacing=labelspacing)  # handletextpad=0.05)  # , labelspacing=0.1)
         ax.get_legend().set_zorder(2000)
     # colorbar
     if styler.cbar and plot_legend:
@@ -203,6 +230,7 @@ def dataset_plot(
     return ax
 
 
+
 def dataset_subplots(
         ds: DataSet,
         shape: Tuple[int, int],
@@ -212,7 +240,7 @@ def dataset_subplots(
         col_vals: List[List[Any]],
         styler: Optional[Styler] = None,
         axs: Optional[np.ndarray] = None,
-        figsize: Tuple[float, float] = (12, 8),
+        figsize: Tuple[float, float] = (9, 6),
         sharex: str = 'col',
         sharey: str = 'row',
         wspace: float = 0.05,
@@ -222,6 +250,7 @@ def dataset_subplots(
         plot_titles: Optional[List[str]] = None,
         subplot_legend: bool = True,
         subplot_cbar: bool = False,
+        subplots_adjust: float = 0.0,
         **kwargs
 ) -> plt.Axes:
     """Plot a dataset as a grid of subplots, split by the 'rows_by' and 'cols_by' columns in the info_table.
@@ -253,13 +282,21 @@ def dataset_subplots(
         fig, axs = plt.subplots(shape[0], shape[1], figsize=figsize, sharex=sharex, sharey=sharey)
         fig.subplots_adjust(wspace=wspace, hspace=hspace)
 
-    if axs.ndim == 1:
+    if shape[0] == 1 and shape[1] == 1:
+        axs = np.array([[axs]])
+    elif shape[0] == 1:
         axs = np.array([axs])
+    elif shape[1] == 1:
+        axs = np.array([[ax] for ax in axs])
 
+    if styler is None:
+        styler = Styler()
+
+    # set the titles of the rows and columns
     if row_titles is not None:
         for ax, row_title in zip(axs[:, 0], row_titles):
             ax.annotate(row_title, xy=(0, 0.5), xytext=(-ax.yaxis.labelpad - 5, 0), xycoords=ax.yaxis.label,
-                        textcoords='offset points', ha='right', va='center', rotation=90)
+                        textcoords='offset points', ha='right', va='center', rotation=90, fontsize=11)
 
     if col_titles is not None:
         for ax, column_title in zip(axs[0, :], col_titles):
@@ -269,16 +306,21 @@ def dataset_subplots(
         for ax, subplot_title in zip(axs.flat, plot_titles):
             ax.set_title(subplot_title)
 
+    # default kwargs
+    if 'plot_legend' not in kwargs:
+        kwargs['plot_legend'] = False
+
     # loop through the grid of axes and plot the subsets
     if rows_by == cols_by:
         for ax, row_val in zip(axs.flat, row_vals):
-            subset = ds[{rows_by: row_val}]
+            subset = ds.subset({rows_by: row_val})
             dataset_plot(subset, styler=styler, ax=ax, **kwargs)
     else:
         for row, row_val in enumerate(row_vals):
             for col, col_val in enumerate(col_vals):
                 ax = axs[row, col]
-                subset = ds[{cols_by: col_val, rows_by: row_val}]
+                subset = ds.subset({cols_by: col_val, rows_by: row_val})
+                if len(subset) == 0: continue
                 dataset_plot(subset, styler=styler, ax=ax, **kwargs)
 
     if subplot_cbar:
@@ -291,13 +333,66 @@ def dataset_subplots(
             cbar.set_label(styler.cbar_label)
 
     if subplot_legend:
-        plt.subplots_adjust(right=0.85)
-        axs.flat[0].get_figure().legend(handles=styler.legend_handles(), loc='center right', frameon=True,
-                                        bbox_to_anchor=(0.925, 0.5), markerfirst=False, handletextpad=0.05)
+        plt.subplots_adjust(right=0.835 + subplots_adjust)
+        axs.flat[0].get_figure().legend(handles=styler.legend_handles(), loc='center right', frameon=False,
+                                        bbox_to_anchor=(0.925, 0.5), markerfirst=True, handletextpad=0.05)
 
     return axs
 
 
+def info_plot(
+        ds: DataSet,
+        x: str,
+        y: str,
+        styler: Optional[Styler] = None,
+        ax: Optional[plt.Axes] = None,
+        plot_legend: bool = True,
+        err_between: Optional[Tuple[str, str]] = None,
+        **kwargs
+) -> plt.Axes:
+    """Make a single combined plot from the info of every dataitem in the dataset using pandas.DataFrame.plot.
+
+    Args:
+        ds: The dataset to plot.
+        x: The column to plot on the x-axis.
+        y: The column to plot on the y-axis.
+        styler: The styler to use for the plot.
+        ax: The axis to plot on.
+        plot_legend: Whether to plot the legend.
+        **kwargs: Additional keyword arguments to pass to the pandas.DataFrame.plot function.
+
+    Returns: The axis the plot was made on.
+    """
+    if ax is None:
+        fig, (ax) = plt.subplots(1, 1, figsize=kwargs.get('figsize', (6, 4)))
+    kwargs['ax'] = ax
+
+    if ax.get_legend() is not None and plot_legend:
+        ax.get_legend().remove()
+
+    kwargs = {**styler.plot_kwargs, **kwargs}
+
+    # plot the dataitems
+    for di in ds:
+        # plot the curve
+        df = pd.DataFrame([[di.info[x], di.info[y]]], columns=[x, y])
+        ax = df.plot(x=x, y=y, **styler.curve_formatters(di), **kwargs)
+        # ax = di.info.plot(x=x, y=y, **styler.curve_formatters(di), **kwargs)
+        if err_between is not None:
+            ax = di.info.plot(x=x, y=y, yerr=[di.info[err_between[0]], di.info[err_between[1]]],
+                              **styler.curve_formatters(di), ax=ax)
+
+    # add the legend
+    handles = styler.legend_handles(ds)
+    if len(handles) > 0 and plot_legend:
+        ax.legend(handles=handles, loc='best', frameon=True, markerfirst=False,
+                  handletextpad=0.05)  # , labelspacing=0.1)
+        ax.get_legend().set_zorder(2000)
+    # colorbar
+
+    return ax
+
+
 def subplot_wrapper(
         ds: DataSet,
         plot_func: Callable[[DataItem, plt.axes], DataItem],
@@ -351,14 +446,14 @@ def subplot_wrapper(
     if rows_by == cols_by:
         for ax, row_val in zip(axs.flat, row_vals):
             kwargs['ax'] = ax
-            subset = ds[{rows_by: row_val}]
+            subset = ds.subset({rows_by: row_val})
             for di in subset:
                 plot_func(di, **kwargs)
     else:
         for row, row_val in enumerate(row_vals):
             for col, col_val in enumerate(col_vals):
                 kwargs['ax'] = axs[row, col]
-                subset = ds[{cols_by: col_val, rows_by: row_val}]
+                subset = ds.subset({cols_by: col_val, rows_by: row_val})
                 for di in subset:
                     plot_func(di, **kwargs)
 
@@ -379,3 +474,38 @@ def subplot_wrapper(
             ax.set_title(subplot_title)
 
     return axs
+
+
+def matrix_plot(
+        ds: DataSet,
+        index: str,
+        columns: str,
+        x_label: str = '',
+        y_label: str = '',
+        titles: Union[str, List[str]] = None,
+        group_by: str = None,
+        group_by_vals: List[str] = None,
+        axs: plt.Axes = None,
+        heatmap_kwargs: Dict[str, Any] = None,
+) -> plt.Axes:
+    if axs is None:
+        fig, axs = plt.subplots(1, len(group_by))
+
+    if group_by is not None:
+        ds_subsets = [ds.subset({group_by: val}) for val in group_by_vals]
+    else:
+        ds_subsets = [ds]
+
+    default_heatmap_kwargs = dict(linewidths=2, cbar=False, annot=True, annot_kws={'size': 10})
+    heatmap_kwargs = default_heatmap_kwargs.update(heatmap_kwargs)
+
+    for sub_ds in ds_subsets:
+        exp_matrix = experimental_matrix(sub_ds.info_table, index=index, columns=columns)
+        sns.heatmap(exp_matrix, **heatmap_kwargs)
+
+    for i, ax in enumerate(axs):
+        ax.set_xlabel(x_label)
+        if titles is not None:
+            axs[i].set_title(titles[i])
+
+    return axs
